#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"

#define RQ_SET_LED    0
#define RQ_GET_SWITCH 1
#define RQ_GET_LIGHT 2

/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */

usbMsgLen_t usbFunctionSetup(uint8_t data[8])
{
    usbRequest_t *rq = (void *)data;
    static uint8_t light_var[2];
    static uint8_t switch_state;  /* must stay valid when usbFunctionSetup returns */
    if (rq->bRequest == RQ_SET_LED)
    {
        uint8_t led_val = rq->wValue.bytes[0];
        uint8_t led_no  = rq->wIndex.bytes[0];

        if (led_val)
            PORTC |= (1<<led_no);
        else
            PORTC &= ~(1<<led_no);
        _delay_ms(500);
        PORTC &= ~(1<<led_no);

        return 0;
    }

    else if (rq->bRequest == RQ_GET_SWITCH)
    {

            uint8_t led_val = rq->wValue.bytes[0];uint8_t led_no = rq->wIndex.bytes[0];
        if ((PINC & (1<<PC3)) == 0) /* switch is pressed */
        {    switch_state = 1;
            PORTC != (1<<led_no);
        }
        else
            switch_state = 0;

        /* point usbMsgPtr to the data to be returned to host */
        usbMsgPtr = (uchar*) &switch_state;

        /* return the number of bytes of data to be returned to host */
        return sizeof(switch_state);
    }

    else if (rq->bRequest == RQ_GET_LIGHT)
    {
        ADMUX = 0b01000000;
        ADMUX |= PC0;
        ADCSRA = 0b11000110;
        while ((ADCSRA & (1<<ADSC)));
        
        light_var[0]= ADCL;
        light_var[1]= ADCH;

        usbMsgPtr = (uchar*) light_var;

        return sizeof(light_var);
    }

    return 0;   /* default for not implemented requests: return no data back to host */
}

/* ------------------------------------------------------------------------- */

int main(void)
{
    DDRC  |=(1<<PC2)|(1<<PC1)|(1<<PC4); // PC0..PC2 = Output, PC3..PC5 = Input
    PORTC = (1 << PC3);                 // Enable pull-up at PC3

    usbInit();

    /* enforce re-enumeration, do this while interrupts are disabled! */
    usbDeviceDisconnect();
    _delay_ms(300);
    usbDeviceConnect();

    /* enable global interrupts */
    sei();

    /* main event loop */
    for(;;)
    {
        usbPoll();

    }

    return 0;
}

/* ------------------------------------------------------------------------- */
